@@ -26,6 +26,8 @@ import java.io.InputStream;
 import java.io.Reader;
 import java.io.UnsupportedEncodingException;
 
+import java.lang.reflect.Constructor;
+import java.lang.reflect.InvocationTargetException;
 import java.math.BigDecimal;
 
 import java.net.URL;
@@ -46,8 +48,10 @@ import java.util.Calendar;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
-import java.util.Locale;
 import java.util.Map;
+import java.util.Properties;
+
+import com.mysql.jdbc.exceptions.NotYetImplementedException;
 
 /**
  * Representation of stored procedures for JDBC
@@ -58,7 +62,38 @@ import java.util.Map;
  */
 public class CallableStatement extends PreparedStatement implements
 		java.sql.CallableStatement {
-	class CallableStatementParam {
+	protected final static Constructor JDBC_4_CSTMT_2_ARGS_CTOR;
+	
+	protected final static Constructor JDBC_4_CSTMT_4_ARGS_CTOR;
+	
+	static {
+		if (Util.isJdbc4()) {
+			try {
+				JDBC_4_CSTMT_2_ARGS_CTOR = Class.forName(
+						"com.mysql.jdbc.JDBC4CallableStatement")
+						.getConstructor(
+								new Class[] { ConnectionImpl.class,
+										CallableStatementParamInfo.class });
+				JDBC_4_CSTMT_4_ARGS_CTOR = Class.forName(
+						"com.mysql.jdbc.JDBC4CallableStatement")
+						.getConstructor(
+								new Class[] { ConnectionImpl.class,
+										String.class, String.class,
+										Boolean.TYPE });
+			} catch (SecurityException e) {
+				throw new RuntimeException(e);
+			} catch (NoSuchMethodException e) {
+				throw new RuntimeException(e);
+			} catch (ClassNotFoundException e) {
+				throw new RuntimeException(e);
+			}
+		} else {
+			JDBC_4_CSTMT_4_ARGS_CTOR = null;
+			JDBC_4_CSTMT_2_ARGS_CTOR = null;
+		}
+	}
+	
+	protected class CallableStatementParam {
 		int desiredJdbcType;
 
 		int index;
@@ -107,7 +142,7 @@ public class CallableStatement extends PreparedStatement implements
 		}
 	}
 
-	class CallableStatementParamInfo {
+	protected class CallableStatementParamInfo {
 		String catalogInUse;
 
 		boolean isFunctionCall;
@@ -179,7 +214,7 @@ public class CallableStatement extends PreparedStatement implements
 			
 			if (isFunctionCall) {
 				this.numParameters += 1;
-			}
+		}
 		}
 
 		private void addParametersFromDBMD(java.sql.ResultSet paramTypesRs)
@@ -274,7 +309,7 @@ public class CallableStatement extends PreparedStatement implements
 			if (this.parameterList == null) {
 				return 0;
 			}
-
+			
 			return this.parameterList.size();
 		}
 
@@ -335,7 +370,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * quite a bit out there in the wild (Websphere, FreeBSD, anyone?)
 	 */
 
-	class CallableStatementParamInfoJDBC3 extends CallableStatementParamInfo
+	protected class CallableStatementParamInfoJDBC3 extends CallableStatementParamInfo
 			implements ParameterMetaData {
 
 		CallableStatementParamInfoJDBC3(java.sql.ResultSet paramTypesRs)
@@ -346,6 +381,54 @@ public class CallableStatement extends PreparedStatement implements
 		public CallableStatementParamInfoJDBC3(CallableStatementParamInfo paramInfo) {
 			super(paramInfo);
 		}
+		
+		/**
+	     * Returns true if this either implements the interface argument or is directly or indirectly a wrapper
+	     * for an object that does. Returns false otherwise. If this implements the interface then return true,
+	     * else if this is a wrapper then return the result of recursively calling <code>isWrapperFor</code> on the wrapped
+	     * object. If this does not implement the interface and is not a wrapper, return false.
+	     * This method should be implemented as a low-cost operation compared to <code>unwrap</code> so that
+	     * callers can use this method to avoid expensive <code>unwrap</code> calls that may fail. If this method
+	     * returns true then calling <code>unwrap</code> with the same argument should succeed.
+	     *
+	     * @param interfaces a Class defining an interface.
+	     * @return true if this implements the interface or directly or indirectly wraps an object that does.
+	     * @throws java.sql.SQLException  if an error occurs while determining whether this is a wrapper
+	     * for an object with the given interface.
+	     * @since 1.6
+	     */
+		public boolean isWrapperFor(Class iface) throws SQLException {
+			checkClosed();
+			
+			// This works for classes that aren't actually wrapping
+			// anything
+			return iface.isInstance(this);
+		}
+
+	    /**
+	     * Returns an object that implements the given interface to allow access to non-standard methods,
+	     * or standard methods not exposed by the proxy.
+	     * The result may be either the object found to implement the interface or a proxy for that object.
+	     * If the receiver implements the interface then that is the object. If the receiver is a wrapper
+	     * and the wrapped object implements the interface then that is the object. Otherwise the object is
+	     *  the result of calling <code>unwrap</code> recursively on the wrapped object. If the receiver is not a
+	     * wrapper and does not implement the interface, then an <code>SQLException</code> is thrown.
+	     *
+	     * @param iface A Class defining an interface that the result must implement.
+	     * @return an object that implements the interface. May be a proxy for the actual implementing object.
+	     * @throws java.sql.SQLException If no object found that implements the interface 
+	     * @since 1.6
+	     */
+		public Object unwrap(Class iface) throws java.sql.SQLException {
+	    	try {
+	    		// This works for classes that aren't actually wrapping
+	    		// anything
+	    		return Util.cast(iface, this);
+	        } catch (ClassCastException cce) {
+	            throw SQLError.createSQLException("Unable to unwrap to " + iface.toString(), 
+	            		SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
+	        }
+	    }
 	}
 
 	private final static int NOT_OUTPUT_PARAMETER_INDICATOR = Integer.MIN_VALUE;
@@ -375,15 +458,15 @@ public class CallableStatement extends PreparedStatement implements
 
 	private boolean callingStoredFunction = false;
 
-	private ResultSet functionReturnValueResults;
+	private ResultSetInternalMethods functionReturnValueResults;
 
 	private boolean hasOutputParams = false;
 
 	// private List parameterList;
 	// private Map parameterMap;
-	private ResultSet outputParameterResults;
+	private ResultSetInternalMethods outputParameterResults;
 
-	private boolean outputParamWasNull = false;
+	protected boolean outputParamWasNull = false;
 
 	private int[] parameterIndexToRsIndex;
 
@@ -402,7 +485,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @throws SQLException
 	 *             if an error occurs
 	 */
-	public CallableStatement(Connection conn,
+	public CallableStatement(ConnectionImpl conn,
 			CallableStatementParamInfo paramInfo) throws SQLException {
 		super(conn, paramInfo.nativeSql, paramInfo.catalogInUse);
 
@@ -415,32 +498,48 @@ public class CallableStatement extends PreparedStatement implements
 	}
 
 	/**
-	 * Creates a new CallableStatement
-	 * 
-	 * @param conn
-	 *            the connection creating this statement
-	 * @param catalog
-	 *            catalog the current catalog
-	 * 
-	 * @throws SQLException
-	 *             if an error occurs
+	 * Creates a callable statement instance -- We need to provide factory-style methods
+	 * so we can support both JDBC3 (and older) and JDBC4 runtimes, otherwise
+	 * the class verifier complains when it tries to load JDBC4-only interface
+	 * classes that are present in JDBC4 method signatures.
 	 */
-	public CallableStatement(Connection conn, String catalog)
-			throws SQLException {
-		super(conn, catalog, null);
 
-		determineParameterTypes();
-		generateParameterMap();
-		
-		if (this.callingStoredFunction) {
-			this.parameterCount += 1;
+	protected static CallableStatement getInstance(ConnectionImpl conn, String sql,
+			String catalog, boolean isFunctionCall) throws SQLException {
+		if (!Util.isJdbc4()) {
+			return new CallableStatement(conn, sql, catalog, isFunctionCall);
 		}
-	}
 
+		return (CallableStatement) Util.handleNewInstance(
+				JDBC_4_CSTMT_4_ARGS_CTOR, new Object[] { conn, sql, catalog,
+						Boolean.valueOf(isFunctionCall) });
+	}
+	
+	/**
+	 * Creates a callable statement instance -- We need to provide factory-style methods
+	 * so we can support both JDBC3 (and older) and JDBC4 runtimes, otherwise
+	 * the class verifier complains when it tries to load JDBC4-only interface
+	 * classes that are present in JDBC4 method signatures.
+	 */
+
+	protected static CallableStatement getInstance(ConnectionImpl conn,
+			CallableStatementParamInfo paramInfo) throws SQLException {
+		if (!Util.isJdbc4()) {
+			return new CallableStatement(conn, paramInfo);
+		}
+
+		return (CallableStatement) Util.handleNewInstance(
+				JDBC_4_CSTMT_2_ARGS_CTOR, new Object[] { conn, paramInfo });
+
+	}
+	
 	private int[] placeholderToParameterIndexMap;
 	
-	
 	private void generateParameterMap() throws SQLException {
+		if (this.paramInfo == null) {
+			return;
+		}
+		
 		// if the user specified some parameters as literals, we need to
 		// provide a map from the specified placeholders to the actual
 		// parameter numbers
@@ -504,7 +603,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @throws SQLException
 	 *             if an error occurs
 	 */
-	public CallableStatement(Connection conn, String sql, String catalog,
+	public CallableStatement(ConnectionImpl conn, String sql, String catalog,
 			boolean isFunctionCall) throws SQLException {
 		super(conn, sql, catalog);
 
@@ -677,7 +776,7 @@ public class CallableStatement extends PreparedStatement implements
 
 			row[12] = null;
 
-			resultRows.add(row);
+			resultRows.add(new ByteArrayRow(row));
 		}
 
 		java.sql.ResultSet paramTypesRs = DatabaseMetaData.buildResultSet(
@@ -880,7 +979,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @throws SQLException
 	 *             if the parameter name is null or empty.
 	 */
-	private String fixParameterName(String paramNameIn) throws SQLException {
+	protected String fixParameterName(String paramNameIn) throws SQLException {
 		if ((paramNameIn == null) || (paramNameIn.length() == 0)) {
 			throw SQLError.createSQLException(
 					((Messages.getString("CallableStatement.0") + paramNameIn) == null) //$NON-NLS-1$
@@ -907,7 +1006,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#getArray(int)
 	 */
 	public synchronized Array getArray(int i) throws SQLException {
-		ResultSet rs = getOutputParameters(i);
+		ResultSetInternalMethods rs = getOutputParameters(i);
 
 		Array retValue = rs.getArray(mapOutputParameterIndexToRsIndex(i));
 
@@ -921,7 +1020,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized Array getArray(String parameterName)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		Array retValue = rs.getArray(fixParameterName(parameterName));
@@ -936,7 +1035,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized BigDecimal getBigDecimal(int parameterIndex)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		BigDecimal retValue = rs
 				.getBigDecimal(mapOutputParameterIndexToRsIndex(parameterIndex));
@@ -964,7 +1063,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized BigDecimal getBigDecimal(int parameterIndex, int scale)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		BigDecimal retValue = rs.getBigDecimal(
 				mapOutputParameterIndexToRsIndex(parameterIndex), scale);
@@ -979,7 +1078,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized BigDecimal getBigDecimal(String parameterName)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		BigDecimal retValue = rs.getBigDecimal(fixParameterName(parameterName));
@@ -993,7 +1092,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#getBlob(int)
 	 */
 	public synchronized Blob getBlob(int parameterIndex) throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		Blob retValue = rs
 				.getBlob(mapOutputParameterIndexToRsIndex(parameterIndex));
@@ -1007,7 +1106,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#getBlob(java.lang.String)
 	 */
 	public synchronized Blob getBlob(String parameterName) throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		Blob retValue = rs.getBlob(fixParameterName(parameterName));
@@ -1022,7 +1121,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized boolean getBoolean(int parameterIndex)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		boolean retValue = rs
 				.getBoolean(mapOutputParameterIndexToRsIndex(parameterIndex));
@@ -1037,7 +1136,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized boolean getBoolean(String parameterName)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		boolean retValue = rs.getBoolean(fixParameterName(parameterName));
@@ -1051,7 +1150,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#getByte(int)
 	 */
 	public synchronized byte getByte(int parameterIndex) throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		byte retValue = rs
 				.getByte(mapOutputParameterIndexToRsIndex(parameterIndex));
@@ -1065,7 +1164,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#getByte(java.lang.String)
 	 */
 	public synchronized byte getByte(String parameterName) throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		byte retValue = rs.getByte(fixParameterName(parameterName));
@@ -1079,7 +1178,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#getBytes(int)
 	 */
 	public synchronized byte[] getBytes(int parameterIndex) throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		byte[] retValue = rs
 				.getBytes(mapOutputParameterIndexToRsIndex(parameterIndex));
@@ -1094,7 +1193,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized byte[] getBytes(String parameterName)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		byte[] retValue = rs.getBytes(fixParameterName(parameterName));
@@ -1108,7 +1207,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#getClob(int)
 	 */
 	public synchronized Clob getClob(int parameterIndex) throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		Clob retValue = rs
 				.getClob(mapOutputParameterIndexToRsIndex(parameterIndex));
@@ -1122,7 +1221,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#getClob(java.lang.String)
 	 */
 	public synchronized Clob getClob(String parameterName) throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		Clob retValue = rs.getClob(fixParameterName(parameterName));
@@ -1136,7 +1235,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#getDate(int)
 	 */
 	public synchronized Date getDate(int parameterIndex) throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		Date retValue = rs
 				.getDate(mapOutputParameterIndexToRsIndex(parameterIndex));
@@ -1151,7 +1250,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized Date getDate(int parameterIndex, Calendar cal)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		Date retValue = rs.getDate(
 				mapOutputParameterIndexToRsIndex(parameterIndex), cal);
@@ -1165,7 +1264,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#getDate(java.lang.String)
 	 */
 	public synchronized Date getDate(String parameterName) throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		Date retValue = rs.getDate(fixParameterName(parameterName));
@@ -1181,7 +1280,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized Date getDate(String parameterName, Calendar cal)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		Date retValue = rs.getDate(fixParameterName(parameterName), cal);
@@ -1196,7 +1295,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized double getDouble(int parameterIndex)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		double retValue = rs
 				.getDouble(mapOutputParameterIndexToRsIndex(parameterIndex));
@@ -1211,7 +1310,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized double getDouble(String parameterName)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		double retValue = rs.getDouble(fixParameterName(parameterName));
@@ -1225,7 +1324,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#getFloat(int)
 	 */
 	public synchronized float getFloat(int parameterIndex) throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		float retValue = rs
 				.getFloat(mapOutputParameterIndexToRsIndex(parameterIndex));
@@ -1240,7 +1339,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized float getFloat(String parameterName)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		float retValue = rs.getFloat(fixParameterName(parameterName));
@@ -1254,7 +1353,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#getInt(int)
 	 */
 	public synchronized int getInt(int parameterIndex) throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		int retValue = rs
 				.getInt(mapOutputParameterIndexToRsIndex(parameterIndex));
@@ -1268,7 +1367,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#getInt(java.lang.String)
 	 */
 	public synchronized int getInt(String parameterName) throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		int retValue = rs.getInt(fixParameterName(parameterName));
@@ -1282,7 +1381,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#getLong(int)
 	 */
 	public synchronized long getLong(int parameterIndex) throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		long retValue = rs
 				.getLong(mapOutputParameterIndexToRsIndex(parameterIndex));
@@ -1296,7 +1395,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#getLong(java.lang.String)
 	 */
 	public synchronized long getLong(String parameterName) throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		long retValue = rs.getLong(fixParameterName(parameterName));
@@ -1306,7 +1405,7 @@ public class CallableStatement extends PreparedStatement implements
 		return retValue;
 	}
 
-	private int getNamedParamIndex(String paramName, boolean forOut)
+	protected int getNamedParamIndex(String paramName, boolean forOut)
 	throws SQLException {
 		if (this.connection.getNoAccessToProcedureBodies()) {
 			throw SQLError.createSQLException("No access to parameters by name when connection has been configured not to access procedure bodies",
@@ -1318,15 +1417,15 @@ public class CallableStatement extends PreparedStatement implements
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		}
 
-		CallableStatementParam namedParamInfo = this.paramInfo
-		.getParameter(paramName);
-
 		if (this.paramInfo == null) {
 			throw SQLError.createSQLException(
 					Messages.getString("CallableStatement.3") + paramName + Messages.getString("CallableStatement.4"), //$NON-NLS-1$ //$NON-NLS-2$
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		}
 
+		CallableStatementParam namedParamInfo = this.paramInfo
+			.getParameter(paramName);
+
 		if (forOut && !namedParamInfo.isOut) {
 			throw SQLError.createSQLException(
 					Messages.getString("CallableStatement.5") + paramName //$NON-NLS-1$
@@ -1356,7 +1455,7 @@ public class CallableStatement extends PreparedStatement implements
 			throws SQLException {
 		CallableStatementParam paramDescriptor = checkIsOutputParam(parameterIndex);
 
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		Object retVal = rs.getObjectStoredProc(
 				mapOutputParameterIndexToRsIndex(parameterIndex),
@@ -1372,7 +1471,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized Object getObject(int parameterIndex, Map map)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		Object retVal = rs.getObject(
 				mapOutputParameterIndexToRsIndex(parameterIndex), map);
@@ -1387,7 +1486,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized Object getObject(String parameterName)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		Object retValue = rs.getObject(fixParameterName(parameterName));
@@ -1403,7 +1502,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized Object getObject(String parameterName, Map map)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		Object retValue = rs.getObject(fixParameterName(parameterName), map);
@@ -1423,7 +1522,7 @@ public class CallableStatement extends PreparedStatement implements
 	 *             if no output parameters were defined, or if no output
 	 *             parameters were returned.
 	 */
-	private ResultSet getOutputParameters(int paramIndex) throws SQLException {
+	protected ResultSetInternalMethods getOutputParameters(int paramIndex) throws SQLException {
 		this.outputParamWasNull = false;
 
 		if (paramIndex == 1 && this.callingStoredFunction
@@ -1448,17 +1547,17 @@ public class CallableStatement extends PreparedStatement implements
 	public synchronized ParameterMetaData getParameterMetaData()
 			throws SQLException {
 		if (this.placeholderToParameterIndexMap == null) {
-			return (CallableStatementParamInfoJDBC3) this.paramInfo;
+		return (CallableStatementParamInfoJDBC3) this.paramInfo;
 		} else {
 			return new CallableStatementParamInfoJDBC3(this.paramInfo);
-		}
+	}
 	}
 
 	/**
 	 * @see java.sql.CallableStatement#getRef(int)
 	 */
 	public synchronized Ref getRef(int parameterIndex) throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		Ref retValue = rs
 				.getRef(mapOutputParameterIndexToRsIndex(parameterIndex));
@@ -1472,7 +1571,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#getRef(java.lang.String)
 	 */
 	public synchronized Ref getRef(String parameterName) throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		Ref retValue = rs.getRef(fixParameterName(parameterName));
@@ -1486,7 +1585,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#getShort(int)
 	 */
 	public synchronized short getShort(int parameterIndex) throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		short retValue = rs
 				.getShort(mapOutputParameterIndexToRsIndex(parameterIndex));
@@ -1501,7 +1600,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized short getShort(String parameterName)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		short retValue = rs.getShort(fixParameterName(parameterName));
@@ -1516,7 +1615,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized String getString(int parameterIndex)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		String retValue = rs
 				.getString(mapOutputParameterIndexToRsIndex(parameterIndex));
@@ -1531,7 +1630,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized String getString(String parameterName)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		String retValue = rs.getString(fixParameterName(parameterName));
@@ -1545,7 +1644,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#getTime(int)
 	 */
 	public synchronized Time getTime(int parameterIndex) throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		Time retValue = rs
 				.getTime(mapOutputParameterIndexToRsIndex(parameterIndex));
@@ -1560,7 +1659,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized Time getTime(int parameterIndex, Calendar cal)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		Time retValue = rs.getTime(
 				mapOutputParameterIndexToRsIndex(parameterIndex), cal);
@@ -1574,7 +1673,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#getTime(java.lang.String)
 	 */
 	public synchronized Time getTime(String parameterName) throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		Time retValue = rs.getTime(fixParameterName(parameterName));
@@ -1590,7 +1689,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized Time getTime(String parameterName, Calendar cal)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		Time retValue = rs.getTime(fixParameterName(parameterName), cal);
@@ -1605,7 +1704,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized Timestamp getTimestamp(int parameterIndex)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		Timestamp retValue = rs
 				.getTimestamp(mapOutputParameterIndexToRsIndex(parameterIndex));
@@ -1620,7 +1719,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized Timestamp getTimestamp(int parameterIndex, Calendar cal)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		Timestamp retValue = rs.getTimestamp(
 				mapOutputParameterIndexToRsIndex(parameterIndex), cal);
@@ -1635,7 +1734,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized Timestamp getTimestamp(String parameterName)
 			throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		Timestamp retValue = rs.getTimestamp(fixParameterName(parameterName));
@@ -1651,7 +1750,7 @@ public class CallableStatement extends PreparedStatement implements
 	 */
 	public synchronized Timestamp getTimestamp(String parameterName,
 			Calendar cal) throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		Timestamp retValue = rs.getTimestamp(fixParameterName(parameterName),
@@ -1666,7 +1765,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#getURL(int)
 	 */
 	public synchronized URL getURL(int parameterIndex) throws SQLException {
-		ResultSet rs = getOutputParameters(parameterIndex);
+		ResultSetInternalMethods rs = getOutputParameters(parameterIndex);
 
 		URL retValue = rs
 				.getURL(mapOutputParameterIndexToRsIndex(parameterIndex));
@@ -1680,7 +1779,7 @@ public class CallableStatement extends PreparedStatement implements
 	 * @see java.sql.CallableStatement#getURL(java.lang.String)
 	 */
 	public synchronized URL getURL(String parameterName) throws SQLException {
-		ResultSet rs = getOutputParameters(0); // definitely not going to be
+		ResultSetInternalMethods rs = getOutputParameters(0); // definitely not going to be
 		// from ?=
 
 		URL retValue = rs.getURL(fixParameterName(parameterName));
@@ -1690,7 +1789,7 @@ public class CallableStatement extends PreparedStatement implements
 		return retValue;
 	}
 
-	private int mapOutputParameterIndexToRsIndex(int paramIndex)
+	protected int mapOutputParameterIndexToRsIndex(int paramIndex)
 			throws SQLException {
 
 		if (this.returnValueParam != null && paramIndex == 1) {
@@ -1830,7 +1929,7 @@ public class CallableStatement extends PreparedStatement implements
 					outParameterStmt = this.connection.createStatement();
 					outParamRs = outParameterStmt
 							.executeQuery(outParameterQuery.toString());
-					this.outputParameterResults = ((com.mysql.jdbc.ResultSet) outParamRs)
+					this.outputParameterResults = ((com.mysql.jdbc.ResultSetInternalMethods) outParamRs)
 							.copy();
 
 					if (!this.outputParameterResults.next()) {
@@ -1962,7 +2061,7 @@ public class CallableStatement extends PreparedStatement implements
 					PreparedStatement setPstmt = null;
 
 					try {
-						setPstmt = this.connection
+						setPstmt = (PreparedStatement) this.connection
 								.clientPrepareStatement(queryBuf.toString());
 
 						byte[] parameterAsBytes = getBytesRepresentation(
@@ -1995,7 +2094,7 @@ public class CallableStatement extends PreparedStatement implements
 								default:
 									// the inherited PreparedStatement methods
 									// have already escaped and quoted these parameters
-									setPstmt.setBytesNoEscape(1, parameterAsBytes); 
+									setPstmt.setBytesNoEscape(1, parameterAsBytes);
 								}
 							}
 						} else {
@@ -2178,4 +2277,75 @@ public class CallableStatement extends PreparedStatement implements
 		
 		return super.getParameterIndexOffset();
 	}
+	
+	public void setAsciiStream(String parameterName, InputStream x) throws SQLException {
+		setAsciiStream(getNamedParamIndex(parameterName, false), x);
+		
+	}
+
+	public void setAsciiStream(String parameterName, InputStream x, long length) throws SQLException {
+		setAsciiStream(getNamedParamIndex(parameterName, false), x, length);
+		
+	}
+
+	public void setBinaryStream(String parameterName, InputStream x) throws SQLException {
+		setBinaryStream(getNamedParamIndex(parameterName, false), x);
+		
+	}
+
+	public void setBinaryStream(String parameterName, InputStream x, long length) throws SQLException {
+		setBinaryStream(getNamedParamIndex(parameterName, false), x, length);
+		
+	}
+
+	public void setBlob(String parameterName, Blob x) throws SQLException {
+		setBlob(getNamedParamIndex(parameterName, false), x);
+		
+	}
+
+	public void setBlob(String parameterName, InputStream inputStream) throws SQLException {
+		setBlob(getNamedParamIndex(parameterName, false), inputStream);
+		
+	}
+
+	public void setBlob(String parameterName, InputStream inputStream, long length) throws SQLException {
+		setBlob(getNamedParamIndex(parameterName, false), inputStream, length);
+		
+	}
+
+	public void setCharacterStream(String parameterName, Reader reader) throws SQLException {
+		setCharacterStream(getNamedParamIndex(parameterName, false), reader);
+		
+	}
+
+	public void setCharacterStream(String parameterName, Reader reader, long length) throws SQLException {
+		setCharacterStream(getNamedParamIndex(parameterName, false), reader, length);
+		
+	}
+
+	public void setClob(String parameterName, Clob x) throws SQLException {
+		setClob(getNamedParamIndex(parameterName, false), x);
+		
+	}
+
+	public void setClob(String parameterName, Reader reader) throws SQLException {
+		setClob(getNamedParamIndex(parameterName, false), reader);
+		
+	}
+
+	public void setClob(String parameterName, Reader reader, long length) throws SQLException {
+		setClob(getNamedParamIndex(parameterName, false), reader, length);
+		
+	}
+
+	public void setNCharacterStream(String parameterName, Reader value) throws SQLException {
+		setNCharacterStream(getNamedParamIndex(parameterName, false), value);
+		
+	}
+
+	public void setNCharacterStream(String parameterName, Reader value, long length) throws SQLException {
+		setNCharacterStream(getNamedParamIndex(parameterName, false), value, length);
+		
+	}
+
 }
