@@ -27,12 +27,13 @@ package com.mysql.jdbc;
 import java.io.UnsupportedEncodingException;
 import java.sql.SQLException;
 import java.sql.Types;
+import java.util.regex.PatternSyntaxException;
 
 /**
  * Field is a class used to describe fields in a ResultSet
  *
  * @author Mark Matthews
- * @version $Id: Field.java 6531 2007-08-30 15:21:06Z mmatthews $
+ * @version $Id: Field.java 6577 2007-09-07 16:12:04Z mmatthews $
  */
 public class Field {
 
@@ -52,7 +53,7 @@ public class Field {
 
 	private String collationName = null;
 
-	private Connection connection = null;
+	private ConnectionImpl connection = null;
 
 	private String databaseName = null;
 
@@ -115,7 +116,7 @@ public class Field {
 	/**
 	 * Constructor used when communicating with 4.1 and newer servers
 	 */
-	Field(Connection conn, byte[] buffer, int databaseNameStart,
+	Field(ConnectionImpl conn, byte[] buffer, int databaseNameStart,
 			int databaseNameLength, int tableNameStart, int tableNameLength,
 			int originalTableNameStart, int originalTableNameLength,
 			int nameStart, int nameLength, int originalColumnNameStart,
@@ -166,8 +167,13 @@ public class Field {
 		        this.mysqlType = MysqlDefs.FIELD_TYPE_VARCHAR;
 		    } else if (this.charsetIndex == 63 ||
 					!this.connection.versionMeetsMinimum(4, 1, 0)) {
-				setBlobTypeBasedOnLength();
-				this.sqlType = MysqlDefs.mysqlToJavaType(this.mysqlType);
+				if (this.connection.getUseBlobToStoreUTF8OutsideBMP() 
+						&& shouldSetupForUtf8StringInBlob()) {
+					setupForUtf8StringInBlob();
+				} else {
+					setBlobTypeBasedOnLength();
+					this.sqlType = MysqlDefs.mysqlToJavaType(this.mysqlType);
+				}
 			} else {
 				// *TEXT masquerading as blob
 				this.mysqlType = MysqlDefs.FIELD_TYPE_VAR_STRING;
@@ -281,10 +287,71 @@ public class Field {
 		}
 	}
 
+	private boolean shouldSetupForUtf8StringInBlob() throws SQLException {
+		String includePattern = this.connection
+				.getUtf8OutsideBmpIncludedColumnNamePattern();
+		String excludePattern = this.connection
+				.getUtf8OutsideBmpExcludedColumnNamePattern();
+
+		if (excludePattern != null
+				&& !StringUtils.isEmptyOrWhitespaceOnly(excludePattern)) {
+			try {
+				if (getOriginalName().matches(excludePattern)) {
+					if (includePattern != null
+							&& !StringUtils.isEmptyOrWhitespaceOnly(includePattern)) {
+						try {
+							if (getOriginalName().matches(includePattern)) {
+								return true;
+							}
+						} catch (PatternSyntaxException pse) {
+							SQLException sqlEx = SQLError
+									.createSQLException(
+											"Illegal regex specified for \"utf8OutsideBmpIncludedColumnNamePattern\"",
+											SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
+
+							if (!this.connection.getParanoid()) {
+								sqlEx.initCause(pse);
+							}
+
+							throw sqlEx;
+						}
+					}
+					
+					return false;
+				}
+			} catch (PatternSyntaxException pse) {
+				SQLException sqlEx = SQLError
+						.createSQLException(
+								"Illegal regex specified for \"utf8OutsideBmpExcludedColumnNamePattern\"",
+								SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
+
+				if (!this.connection.getParanoid()) {
+					sqlEx.initCause(pse);
+				}
+
+				throw sqlEx;
+			}
+		}
+
+		return true;
+	}
+
+	private void setupForUtf8StringInBlob() {
+		if (this.length == MysqlDefs.LENGTH_TINYBLOB || this.length == MysqlDefs.LENGTH_BLOB) {
+			this.mysqlType = MysqlDefs.FIELD_TYPE_VARCHAR;
+			this.sqlType = Types.VARCHAR;
+		}  else {
+			this.mysqlType = MysqlDefs.FIELD_TYPE_VAR_STRING;
+			this.sqlType = Types.LONGVARCHAR;
+		}
+		
+		this.charsetIndex = 33;	
+	}
+
 	/**
 	 * Constructor used when communicating with pre 4.1 servers
 	 */
-	Field(Connection conn, byte[] buffer, int nameStart, int nameLength,
+	Field(ConnectionImpl conn, byte[] buffer, int nameStart, int nameLength,
 			int tableNameStart, int tableNameLength, int length, int mysqlType,
 			short colFlag, int colDecimals) throws SQLException {
 		this(conn, buffer, -1, -1, tableNameStart, tableNameLength, -1, -1,
@@ -303,7 +370,34 @@ public class Field {
 		this.colFlag = 0;
 		this.colDecimals = 0;
 	}
-
+	
+	/**
+	 * Used by prepared statements to re-use result set data conversion methods
+	 * when generating bound parmeter retrieval instance for statement
+	 * interceptors.
+	 * 
+	 * @param tableName
+	 *            not used
+	 * @param columnName
+	 *            not used
+	 * @param charsetIndex
+	 *            the MySQL collation/character set index
+	 * @param jdbcType
+	 *            from java.sql.Types
+	 * @param length
+	 *            length in characters or bytes (for BINARY data).
+	 */
+	Field(String tableName, String columnName, int charsetIndex, int jdbcType,
+			int length) {
+		this.tableName = tableName;
+		this.name = columnName;
+		this.length = length;
+		this.sqlType = jdbcType;
+		this.colFlag = 0;
+		this.colDecimals = 0;
+		this.charsetIndex = charsetIndex;
+	}
+	
 	private void checkForImplicitTemporaryTable() {
 		this.isImplicitTempTable = this.tableNameLength > 5
 				&& this.buffer[tableNameStart] == (byte) '#'
@@ -395,7 +489,7 @@ public class Field {
 
 		return this.collationName;
 	}
-
+	
 	public String getColumnLabel() throws SQLException {
 		return getName(); // column name if not aliased, alias if used
 	}
@@ -841,7 +935,7 @@ public class Field {
 	 * @param conn
 	 *            DOCUMENT ME!
 	 */
-	public void setConnection(Connection conn) {
+	public void setConnection(ConnectionImpl conn) {
 		this.connection = conn;
 
 		this.charsetName = this.connection.getEncoding();
@@ -858,31 +952,72 @@ public class Field {
 
 	public String toString() {
 		try {
-			StringBuffer asString = new StringBuffer(128);
+			StringBuffer asString = new StringBuffer();
 			asString.append(super.toString());
-
-			asString.append("\n  catalog: ");
+			asString.append("[");
+			asString.append("catalog=");
 			asString.append(this.getDatabaseName());
-			asString.append("\n  table name: ");
+			asString.append(",tableName=");
 			asString.append(this.getTableName());
-			asString.append("\n  original table name: ");
+			asString.append(",originalTableName=");
 			asString.append(this.getOriginalTableName());
-			asString.append("\n  column name: ");
+			asString.append(",columnName=");
 			asString.append(this.getName());
-			asString.append("\n  original column name: ");
+			asString.append(",originalColumnName=");
 			asString.append(this.getOriginalName());
-			asString.append("\n  MySQL data type: ");
+			asString.append(",mysqlType=");
 			asString.append(getMysqlType());
 			asString.append("(");
 			asString.append(MysqlDefs.typeToName(getMysqlType()));
 			asString.append(")");
-
-			if (this.buffer != null) {
-				asString.append("\n\nData as received from server:\n\n");
-				asString.append(StringUtils.dumpAsHex(this.buffer,
-						this.buffer.length));
+			asString.append(",flags=");
+			
+			if (isAutoIncrement()) {
+				asString.append(" AUTO_INCREMENT");
+			}
+			
+			if (isPrimaryKey()) {
+				asString.append(" PRIMARY_KEY");
+			}
+			
+			if (isUniqueKey()) {
+				asString.append(" UNIQUE_KEY");
+			}
+			
+			if (isBinary()) {
+				asString.append(" BINARY");
+			}
+			
+			if (isBlob()) {
+				asString.append(" BLOB");
+			}
+			
+			if (isMultipleKey()) {
+				asString.append(" MULTI_KEY");
+			}
+			
+			if (isUnsigned()) {
+				asString.append(" UNSIGNED");
+			}
+			
+			if (isZeroFill()) {
+				asString.append(" ZEROFILL");
 			}
 
+			asString.append(", charsetIndex=");
+			asString.append(this.charsetIndex);
+			asString.append(", charsetName=");
+			asString.append(this.charsetName);
+			
+			
+			//if (this.buffer != null) {
+			//	asString.append("\n\nData as received from server:\n\n");
+			//	asString.append(StringUtils.dumpAsHex(this.buffer,
+			//			this.buffer.length));
+			//}
+
+			asString.append("]");
+			
 			return asString.toString();
 		} catch (Throwable t) {
 			return super.toString();
