@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2002-2004 MySQL AB
+ Copyright (C) 2002-2007 MySQL AB
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of version 2 of the GNU General Public License as 
@@ -24,17 +24,18 @@
  */
 package com.mysql.jdbc;
 
-import com.mysql.jdbc.Statement.CancelTask;
+import com.mysql.jdbc.exceptions.MySQLStatementCancelledException;
 import com.mysql.jdbc.exceptions.MySQLTimeoutException;
 import com.mysql.jdbc.profiler.ProfileEventSink;
 import com.mysql.jdbc.profiler.ProfilerEvent;
 
-import java.io.ByteArrayInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.Reader;
 import java.io.UnsupportedEncodingException;
 
+import java.lang.reflect.Constructor;
+import java.lang.reflect.InvocationTargetException;
 import java.math.BigDecimal;
 
 import java.net.URL;
@@ -45,15 +46,15 @@ import java.sql.Clob;
 import java.sql.Date;
 import java.sql.ParameterMetaData;
 import java.sql.Ref;
+import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Time;
 import java.sql.Timestamp;
 import java.sql.Types;
 
 import java.util.ArrayList;
-import java.util.BitSet;
 import java.util.Calendar;
-import java.util.Locale;
+import java.util.Properties;
 import java.util.TimeZone;
 
 /**
@@ -64,6 +65,27 @@ import java.util.TimeZone;
  *          mmatthews Exp $
  */
 public class ServerPreparedStatement extends PreparedStatement {
+	private static final Constructor JDBC_4_SPS_CTOR;
+	
+	static {
+		if (Util.isJdbc4()) {
+			try {
+				JDBC_4_SPS_CTOR = Class.forName("com.mysql.jdbc.JDBC4ServerPreparedStatement")
+				.getConstructor(
+				new Class[] { ConnectionImpl.class, String.class, String.class,
+						Integer.TYPE, Integer.TYPE});
+			} catch (SecurityException e) {
+				throw new RuntimeException(e);
+			} catch (NoSuchMethodException e) {
+				throw new RuntimeException(e);
+			} catch (ClassNotFoundException e) {
+				throw new RuntimeException(e);
+			} 
+		} else {
+			JDBC_4_SPS_CTOR = null;
+		}
+	}
+	
 	protected static final int BLOB_STREAM_READ_BUF_SIZE = 8192;
 
 	static class BatchedBindValues {
@@ -80,11 +102,11 @@ public class ServerPreparedStatement extends PreparedStatement {
 		}
 	}
 
-	static class BindValue {
+	public static class BindValue {
 
 		long boundBeforeExecutionNum = 0;
 		
-		long bindLength; /* Default length of data */
+		public long bindLength; /* Default length of data */
 
 		int bufferType; /* buffer type */
 
@@ -96,9 +118,9 @@ public class ServerPreparedStatement extends PreparedStatement {
 
 		int intBinding;
 
-		boolean isLongData; /* long data indicator */
+		public boolean isLongData; /* long data indicator */
 
-		boolean isNull; /* NULL indicator */
+		public boolean isNull; /* NULL indicator */
 
 		boolean isSet = false; /* has this parameter been set? */
 
@@ -106,7 +128,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 
 		short shortBinding;
 
-		Object value; /* The value to store */
+		public Object value; /* The value to store */
 
 		BindValue() {
 		}
@@ -271,6 +293,42 @@ public class ServerPreparedStatement extends PreparedStatement {
 
 	private boolean serverNeedsResetBeforeEachExecution;
 
+	/**
+	 * Creates a prepared statement instance -- We need to provide factory-style
+	 * methods so we can support both JDBC3 (and older) and JDBC4 runtimes,
+	 * otherwise the class verifier complains when it tries to load JDBC4-only
+	 * interface classes that are present in JDBC4 method signatures.
+	 */
+
+	protected static ServerPreparedStatement getInstance(ConnectionImpl conn,
+			String sql, String catalog, int resultSetType,
+			int resultSetConcurrency) throws SQLException {
+		if (!Util.isJdbc4()) {
+			return new ServerPreparedStatement(conn, sql, catalog,
+					resultSetType, resultSetConcurrency);
+		}
+
+		try {
+			return (ServerPreparedStatement) JDBC_4_SPS_CTOR.newInstance(new Object[] { conn,
+					sql, catalog, Constants.integerValueOf(resultSetType),
+					Constants.integerValueOf(resultSetConcurrency) });
+		} catch (IllegalArgumentException e) {
+			throw new SQLException(e.toString(), SQLError.SQL_STATE_GENERAL_ERROR);
+		} catch (InstantiationException e) {
+			throw new SQLException(e.toString(), SQLError.SQL_STATE_GENERAL_ERROR);
+		} catch (IllegalAccessException e) {
+			throw new SQLException(e.toString(), SQLError.SQL_STATE_GENERAL_ERROR);
+		} catch (InvocationTargetException e) {
+			Throwable target = e.getTargetException(); 
+			
+			if (target instanceof SQLException) {
+				throw (SQLException)target;
+			}
+			
+			throw new SQLException(target.toString(), SQLError.SQL_STATE_GENERAL_ERROR);
+		}
+	}
+
 	/**
 	 * Creates a new ServerPreparedStatement object.
 	 * 
@@ -284,15 +342,18 @@ public class ServerPreparedStatement extends PreparedStatement {
 	 * @throws SQLException
 	 *             If an error occurs
 	 */
-	public ServerPreparedStatement(Connection conn, String sql, String catalog,
+	protected ServerPreparedStatement(ConnectionImpl conn, String sql, String catalog,
 			int resultSetType, int resultSetConcurrency)
 			throws SQLException {
 		super(conn, catalog);
 
 		checkNullOrEmptyQuery(sql);
 
-		this.isSelectQuery = StringUtils.startsWithIgnoreCaseAndWs(sql,
-				"SELECT"); //$NON-NLS-1$
+		int startOfStatement = findStartOfStatement(sql);
+		
+		this.firstCharOfStmt = StringUtils.firstAlphaCharUc(sql, startOfStatement);
+		
+		this.isSelectQuery = 'S' == this.firstCharOfStmt;
 		
 		if (this.connection.versionMeetsMinimum(5, 0, 0)) {
 			this.serverNeedsResetBeforeEachExecution = 
@@ -302,10 +363,14 @@ public class ServerPreparedStatement extends PreparedStatement {
 				!this.connection.versionMeetsMinimum(4, 1, 10);
 		}
 		
+		this.useAutoSlowLog = this.connection.getAutoSlowLog();
 		this.useTrueBoolean = this.connection.versionMeetsMinimum(3, 21, 23);
 		this.hasLimitClause = (StringUtils.indexOfIgnoreCase(sql, "LIMIT") != -1); //$NON-NLS-1$
-		this.firstCharOfStmt = StringUtils.firstNonWsCharUc(sql);
-		this.originalSql = sql;
+		
+		String statementComment = this.connection.getStatementComment();
+
+		this.originalSql = (statementComment == null) ? sql : "/* "
+				+ statementComment + " */ " + sql;
 
 		if (this.connection.versionMeetsMinimum(4, 1, 2)) {
 			this.stringTypeCode = MysqlDefs.FIELD_TYPE_VAR_STRING;
@@ -328,6 +393,8 @@ public class ServerPreparedStatement extends PreparedStatement {
 		
 		setResultSetType(resultSetType);
 		setResultSetConcurrency(resultSetConcurrency);
+		
+		this.parameterTypes = new int[this.parameterCount];
 	}
 
 	/**
@@ -336,7 +403,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 	 * @exception SQLException
 	 *                if a database-access error occurs.
 	 * 
-	 * @see Statement#addBatch
+	 * @see StatementImpl#addBatch
 	 */
 	public synchronized void addBatch() throws SQLException {
 		checkClosed();
@@ -357,7 +424,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 		PreparedStatement pStmtForSub = null;
 
 		try {
-			pStmtForSub = new PreparedStatement(this.connection,
+			pStmtForSub = PreparedStatement.getInstance(this.connection,
 					this.originalSql, this.currentCatalog);
 
 			int numParameters = pStmtForSub.parameterCount;
@@ -460,6 +527,8 @@ public class ServerPreparedStatement extends PreparedStatement {
 
 	protected boolean isCached = false;
 
+	private boolean useAutoSlowLog;
+
 	protected void setClosed(boolean flag) {
 		this.isClosed = flag;
 	}
@@ -634,8 +703,8 @@ public class ServerPreparedStatement extends PreparedStatement {
 
 										while (rs.next()) {
 											this.batchedGeneratedKeys
-													.add(new byte[][] { rs
-															.getBytes(1) });
+													.add(new ByteArrayRow(new byte[][] { rs
+															.getBytes(1) }));
 										}
 									} finally {
 										if (rs != null) {
@@ -682,15 +751,17 @@ public class ServerPreparedStatement extends PreparedStatement {
 	 * @see com.mysql.jdbc.PreparedStatement#executeInternal(int,
 	 *      com.mysql.jdbc.Buffer, boolean, boolean)
 	 */
-	protected com.mysql.jdbc.ResultSet executeInternal(int maxRowsToRetrieve,
+	protected com.mysql.jdbc.ResultSetInternalMethods executeInternal(int maxRowsToRetrieve,
 			Buffer sendPacket, boolean createStreamingResultSet,
-			boolean queryIsSelectOnly, boolean unpackFields, boolean isBatch)
+			boolean queryIsSelectOnly, Field[] metadataFromCache,
+			boolean isBatch)
 			throws SQLException {
 		this.numberOfExecutions++;
 
 		// We defer to server-side execution
 		try {
-			return serverExecute(maxRowsToRetrieve, createStreamingResultSet);
+			return serverExecute(maxRowsToRetrieve, createStreamingResultSet, 
+					metadataFromCache);
 		} catch (SQLException sqlEx) {
 			// don't wrap SQLExceptions
 			if (this.connection.getEnablePacketDebug()) {
@@ -705,7 +776,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 						.append("\n\nQuery being executed when exception was thrown:\n\n");
 				messageBuf.append(extractedSql);
 
-				sqlEx = Connection.appendMessageToException(sqlEx, messageBuf
+				sqlEx = ConnectionImpl.appendMessageToException(sqlEx, messageBuf
 						.toString());
 			}
 
@@ -726,7 +797,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 						.append("\n\nQuery being executed when exception was thrown:\n\n");
 				messageBuf.append(extractedSql);
 
-				sqlEx = Connection.appendMessageToException(sqlEx, messageBuf
+				sqlEx = ConnectionImpl.appendMessageToException(sqlEx, messageBuf
 						.toString());
 			}
 
@@ -751,7 +822,16 @@ public class ServerPreparedStatement extends PreparedStatement {
 		return null; // we don't use this type of packet
 	}
 
-	private BindValue getBinding(int parameterIndex, boolean forLongData)
+	/**
+	 * Returns the structure representing the value that (can be)/(is)
+	 * bound at the given parameter index.
+	 * 
+	 * @param parameterIndex 1-based
+	 * @param forLongData is this for a stream?
+	 * @return
+	 * @throws SQLException
+	 */
+	protected BindValue getBinding(int parameterIndex, boolean forLongData)
 			throws SQLException {
 		checkClosed();
 		
@@ -878,52 +958,49 @@ public class ServerPreparedStatement extends PreparedStatement {
 			if (this.connection.getAutoGenerateTestcaseScript()) {
 				dumpCloseForTestcase();
 			}
-			
-			synchronized (this.connection.getMutex()) {
-	
-				//
-				// Don't communicate with the server if we're being
-				// called from the finalizer...
-				// 
-				// This will leak server resources, but if we don't do this,
-				// we'll deadlock (potentially, because there's no guarantee
-				// when, what order, and what concurrency finalizers will be
-				// called with). Well-behaved programs won't rely on finalizers
-				// to clean up their statements.
-				//
-				
-				SQLException exceptionDuringClose = null;
-				
 
-				if (calledExplicitly) {
+			//
+			// Don't communicate with the server if we're being
+			// called from the finalizer...
+			// 
+			// This will leak server resources, but if we don't do this,
+			// we'll deadlock (potentially, because there's no guarantee
+			// when, what order, and what concurrency finalizers will be
+			// called with). Well-behaved programs won't rely on finalizers
+			// to clean up their statements.
+			//
+
+			SQLException exceptionDuringClose = null;
+
+			if (calledExplicitly && !this.connection.isClosed()) {
+				synchronized (this.connection.getMutex()) {
 					try {
-						
+
 						MysqlIO mysql = this.connection.getIO();
-						
+
 						Buffer packet = mysql.getSharedSendPacket();
-						
+
 						packet.writeByte((byte) MysqlDefs.COM_CLOSE_STATEMENT);
 						packet.writeLong(this.serverStatementId);
-						
+
 						mysql.sendCommand(MysqlDefs.COM_CLOSE_STATEMENT, null,
 								packet, true, null);
 					} catch (SQLException sqlEx) {
 						exceptionDuringClose = sqlEx;
 					}
-					
 				}
+			}
 
-				super.realClose(calledExplicitly, closeOpenResults);
+			super.realClose(calledExplicitly, closeOpenResults);
 
-				clearParametersInternal(false);
-				this.parameterBindings = null;
-				
-				this.parameterFields = null;
-				this.resultFields = null;
-				
-				if (exceptionDuringClose != null) {
-					throw exceptionDuringClose;
-				}
+			clearParametersInternal(false);
+			this.parameterBindings = null;
+
+			this.parameterFields = null;
+			this.resultFields = null;
+
+			if (exceptionDuringClose != null) {
+				throw exceptionDuringClose;
 			}
 		}
 	}
@@ -1009,8 +1086,9 @@ public class ServerPreparedStatement extends PreparedStatement {
 	 * 
 	 * @throws SQLException
 	 */
-	private com.mysql.jdbc.ResultSet serverExecute(int maxRowsToRetrieve,
-			boolean createStreamingResultSet) throws SQLException {
+	private com.mysql.jdbc.ResultSetInternalMethods serverExecute(int maxRowsToRetrieve,
+			boolean createStreamingResultSet, 
+			Field[] metadataFromCache) throws SQLException {
 		synchronized (this.connection.getMutex()) {
 			if (this.detectedLongParameterSwitch) {
 				// Check when values were bound
@@ -1148,20 +1226,23 @@ public class ServerPreparedStatement extends PreparedStatement {
 
 			long begin = 0;
 
-			if (this.connection.getProfileSql()
-					|| this.connection.getLogSlowQueries()
-					|| this.connection.getGatherPerformanceMetrics()) {
-				begin = System.currentTimeMillis();
+			boolean logSlowQueries = this.connection.getLogSlowQueries();
+			boolean gatherPerformanceMetrics = this.connection
+					.getGatherPerformanceMetrics();
+
+			if (this.profileSQL || logSlowQueries || gatherPerformanceMetrics) {
+				begin = mysql.getCurrentTimeNanosOrMillis();
 			}
 
-			this.wasCancelled = false;
+			resetCancelledState();
 			
 			CancelTask timeoutTask = null;
 
 			try {
-				if (this.timeoutInMillis != 0
+				if (this.connection.getEnableQueryTimeouts() &&
+						this.timeoutInMillis != 0
 						&& this.connection.versionMeetsMinimum(5, 0, 0)) {
-					timeoutTask = new CancelTask();
+					timeoutTask = new CancelTask(this);
 					this.connection.getCancelTimer().schedule(timeoutTask, 
 							this.timeoutInMillis);
 				}
@@ -1169,83 +1250,136 @@ public class ServerPreparedStatement extends PreparedStatement {
 				Buffer resultPacket = mysql.sendCommand(MysqlDefs.COM_EXECUTE,
 					null, packet, false, null);
 				
+				long queryEndTime = 0L;
+				
+				if (logSlowQueries || gatherPerformanceMetrics || this.profileSQL) {
+					queryEndTime = mysql.getCurrentTimeNanosOrMillis();
+				}
+				
 				if (timeoutTask != null) {
 					timeoutTask.cancel();
+					
+					if (timeoutTask.caughtWhileCancelling != null) {
+						throw timeoutTask.caughtWhileCancelling;
+					}
+					
 					timeoutTask = null;
 				}
 				
-				if (this.wasCancelled) {
-					this.wasCancelled = false;
-					throw new MySQLTimeoutException();
+				synchronized (this.cancelTimeoutMutex) {
+					if (this.wasCancelled) {
+						SQLException cause = null;
+						
+						if (this.wasCancelledByTimeout) {
+							cause = new MySQLTimeoutException();
+						} else {
+							cause = new MySQLStatementCancelledException();
+						}
+						
+						resetCancelledState();
+						
+						throw cause;
+					}
 				}
-			
+
+				boolean queryWasSlow = false;
+				
+				if (logSlowQueries || gatherPerformanceMetrics) {
+					long elapsedTime = queryEndTime - begin;
+
+					if (logSlowQueries) {
+		    			if (this.useAutoSlowLog) {
+		    				queryWasSlow = elapsedTime > this.connection.getSlowQueryThresholdMillis();
+		    			} else {
+		    				queryWasSlow = this.connection.isAbonormallyLongQuery(elapsedTime);
+		    				
+		    				this.connection.reportQueryTime(elapsedTime);
+		    			}
+					}
+
+					if (queryWasSlow) {
+						
+						StringBuffer mesgBuf = new StringBuffer(
+								48 + this.originalSql.length());
+						mesgBuf.append(Messages
+								.getString("ServerPreparedStatement.15")); //$NON-NLS-1$
+						mesgBuf.append(mysql.getSlowQueryThreshold());
+						mesgBuf.append(Messages
+								.getString("ServerPreparedStatement.15a")); //$NON-NLS-1$
+						mesgBuf.append(elapsedTime);
+						mesgBuf.append(Messages
+								.getString("ServerPreparedStatement.16")); //$NON-NLS-1$
+
+						mesgBuf.append("as prepared: ");
+						mesgBuf.append(this.originalSql);
+						mesgBuf.append("\n\n with parameters bound:\n\n");
+						mesgBuf.append(asSql(true));
+
+						this.eventSink
+								.consumeEvent(new ProfilerEvent(
+										ProfilerEvent.TYPE_SLOW_QUERY,
+										"", this.currentCatalog, this.connection.getId(), //$NON-NLS-1$
+										getId(), 0, System.currentTimeMillis(),
+										elapsedTime, mysql
+												.getQueryTimingUnits(), null,
+										new Throwable(), mesgBuf.toString()));
+					}
+
+					if (gatherPerformanceMetrics) {
+						this.connection.registerQueryExecutionTime(elapsedTime);
+					}
+				}
+
 				this.connection.incrementNumberOfPreparedExecutes();
-	
-				if (this.connection.getProfileSql()) {
-					this.eventSink = ProfileEventSink.getInstance(this.connection);
-	
+
+				if (this.profileSQL) {
+					this.eventSink = ProfileEventSink
+							.getInstance(this.connection);
+
 					this.eventSink.consumeEvent(new ProfilerEvent(
-							ProfilerEvent.TYPE_EXECUTE, "", this.currentCatalog, //$NON-NLS-1$
+							ProfilerEvent.TYPE_EXECUTE,
+							"", this.currentCatalog, //$NON-NLS-1$
 							this.connectionId, this.statementId, -1, System
-									.currentTimeMillis(), (int) (System
-									.currentTimeMillis() - begin), null,
-							new Throwable(), truncateQueryToLog(asSql(true))));
+									.currentTimeMillis(), (int) (mysql
+									.getCurrentTimeNanosOrMillis() - begin),
+							mysql.getQueryTimingUnits(), null, new Throwable(),
+							truncateQueryToLog(asSql(true))));
 				}
 	
-				com.mysql.jdbc.ResultSet rs = mysql.readAllResults(this,
+				com.mysql.jdbc.ResultSetInternalMethods rs = mysql.readAllResults(this,
 						maxRowsToRetrieve, this.resultSetType,
 						this.resultSetConcurrency, createStreamingResultSet,
 						this.currentCatalog, resultPacket, true, this.fieldCount,
-						true);
+						metadataFromCache);
+				
+				if (this.profileSQL) {
+					long fetchEndTime = mysql.getCurrentTimeNanosOrMillis();
+
+					this.eventSink.consumeEvent(new ProfilerEvent(
+							ProfilerEvent.TYPE_FETCH,
+							"", this.currentCatalog, this.connection.getId(), //$NON-NLS-1$
+							getId(), 0 /* FIXME rs.resultId */, System.currentTimeMillis(),
+							(fetchEndTime - queryEndTime), mysql
+									.getQueryTimingUnits(), null,
+							new Throwable(), null));
+				}
 	
+				if (queryWasSlow && this.connection.getExplainSlowQueries()) {
+					String queryAsString = asSql(true);
+
+					mysql.explainSlowQuery(queryAsString.getBytes(),
+							queryAsString);
+				}
 				
 				if (!createStreamingResultSet && 
 						this.serverNeedsResetBeforeEachExecution) {
 					serverResetStatement(); // clear any long data...
 				}
 	
+				
 				this.sendTypesToServer = false;
 				this.results = rs;
 	
-				if (this.connection.getLogSlowQueries()
-						|| this.connection.getGatherPerformanceMetrics()) {
-					long elapsedTime = System.currentTimeMillis() - begin;
-	
-					if (this.connection.getLogSlowQueries()
-							&& (elapsedTime >= this.connection
-									.getSlowQueryThresholdMillis())) {
-						StringBuffer mesgBuf = new StringBuffer(
-								48 + this.originalSql.length());
-						mesgBuf.append(Messages
-								.getString("ServerPreparedStatement.15")); //$NON-NLS-1$
-						mesgBuf.append(this.connection
-								.getSlowQueryThresholdMillis());
-						mesgBuf.append(Messages
-								.getString("ServerPreparedStatement.15a")); //$NON-NLS-1$
-						mesgBuf.append(elapsedTime);
-						mesgBuf.append(Messages
-								.getString("ServerPreparedStatement.16")); //$NON-NLS-1$
-						
-						mesgBuf.append("as prepared: ");
-						mesgBuf.append(this.originalSql);
-						mesgBuf.append("\n\n with parameters bound:\n\n");
-						mesgBuf.append(asSql(true));
-	
-						this.connection.getLog().logWarn(mesgBuf.toString());
-	
-						if (this.connection.getExplainSlowQueries()) {
-							String queryAsString = asSql(true);
-	
-							mysql.explainSlowQuery(queryAsString.getBytes(),
-									queryAsString);
-						}
-					}
-	
-					if (this.connection.getGatherPerformanceMetrics()) {
-						this.connection.registerQueryExecutionTime(elapsedTime);
-					}
-				}
-				
 				if (mysql.hadWarnings()) {
 		            mysql.scanForAndThrowDataTruncation();
 		        }
@@ -1259,7 +1393,6 @@ public class ServerPreparedStatement extends PreparedStatement {
 		}
 	}
 
-
 	/**
 	 * Sends stream-type data parameters to the server.
 	 * 
@@ -1378,16 +1511,14 @@ public class ServerPreparedStatement extends PreparedStatement {
 
 				this.connection.incrementNumberOfPrepares();
 
-				if (this.connection.getProfileSql()) {
-					this.eventSink = ProfileEventSink
-							.getInstance(this.connection);
-
+				if (this.profileSQL) {
 					this.eventSink.consumeEvent(new ProfilerEvent(
 							ProfilerEvent.TYPE_PREPARE,
 							"", this.currentCatalog, //$NON-NLS-1$
 							this.connectionId, this.statementId, -1,
-							System.currentTimeMillis(), (int) (System
-									.currentTimeMillis() - begin), null,
+							System.currentTimeMillis(), 
+							mysql.getCurrentTimeNanosOrMillis() - begin, 
+							mysql.getQueryTimingUnits(), null,
 							new Throwable(), truncateQueryToLog(sql)));
 				}
 
@@ -1432,7 +1563,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 							.append("\n\nQuery being prepared when exception was thrown:\n\n");
 					messageBuf.append(this.originalSql);
 
-					sqlEx = Connection.appendMessageToException(sqlEx,
+					sqlEx = ConnectionImpl.appendMessageToException(sqlEx,
 							messageBuf.toString());
 				}
 
@@ -1540,7 +1671,8 @@ public class ServerPreparedStatement extends PreparedStatement {
 				setType(binding, this.stringTypeCode);
 			}
 
-			binding.value = StringUtils.fixDecimalExponent(x.toString());
+			binding.value = StringUtils
+				.fixDecimalExponent(StringUtils.consistentToString(x));
 			binding.isNull = false;
 			binding.isLongData = false;
 		}
@@ -1896,7 +2028,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 	 */
 	public void setTime(int parameterIndex, java.sql.Time x)
 			throws SQLException {
-		setTimeInternal(parameterIndex, x, null, TimeZone.getDefault(), false);
+		setTimeInternal(parameterIndex, x, null, this.connection.getDefaultTimeZone(), false);
 	}
 
 	/**
@@ -1973,7 +2105,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 	 */
 	public void setTimestamp(int parameterIndex, java.sql.Timestamp x)
 			throws SQLException {
-		setTimestampInternal(parameterIndex, x, null, TimeZone.getDefault(), false);
+		setTimestampInternal(parameterIndex, x, null, this.connection.getDefaultTimeZone(), false);
 	}
 
 	/**
@@ -2023,7 +2155,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 		}
 	}
 
-	private void setType(BindValue oldValue, int bufferType) {
+	protected void setType(BindValue oldValue, int bufferType) {
 		if (oldValue.bufferType != bufferType) {
 			this.sendTypesToServer = true;
 		}
@@ -2217,12 +2349,12 @@ public class ServerPreparedStatement extends PreparedStatement {
 
 				byte length = (byte) 7;
 
-				intoBuf.ensureCapacity(length);
-
 				if (dt instanceof java.sql.Timestamp) {
 					length = (byte) 11;
 				}
 
+				intoBuf.ensureCapacity(length);
+				
 				intoBuf.writeByte(length); // length
 
 				int year = sessionCalendar.get(Calendar.YEAR);
@@ -2247,7 +2379,8 @@ public class ServerPreparedStatement extends PreparedStatement {
 				}
 
 				if (length == 11) {
-					intoBuf.writeLong(((java.sql.Timestamp) dt).getNanos());
+					//	MySQL expects microseconds, not nanos
+					intoBuf.writeLong(((java.sql.Timestamp) dt).getNanos() / 1000);
 				}
 			
 			} finally {
@@ -2442,4 +2575,6 @@ public class ServerPreparedStatement extends PreparedStatement {
 	protected long getServerStatementId() {
 		return serverStatementId;
 	}
+
+
 }
