@@ -143,8 +143,10 @@ public class StringUtils {
 			outputBuf.append("    "); //$NON-NLS-1$
 
 			for (int j = 0; j < 8; j++) {
-				if ((byteBuffer[p] > 32) && (byteBuffer[p] < 127)) {
-					outputBuf.append((char) byteBuffer[p] + " "); //$NON-NLS-1$
+				int b = 0xff & byteBuffer[p];
+				
+				if (b > 32 && b < 127) {
+					outputBuf.append((char) b + " "); //$NON-NLS-1$
 				} else {
 					outputBuf.append(". "); //$NON-NLS-1$
 				}
@@ -175,8 +177,10 @@ public class StringUtils {
 		outputBuf.append("    "); //$NON-NLS-1$
 
 		for (int i = p; i < length; i++) {
-			if ((byteBuffer[i] > 32) && (byteBuffer[i] < 127)) {
-				outputBuf.append((char) byteBuffer[i] + " "); //$NON-NLS-1$
+			int b = 0xff & byteBuffer[i];
+			
+			if (b > 32 && b < 127) {
+				outputBuf.append((char) b + " "); //$NON-NLS-1$
 			} else {
 				outputBuf.append(". "); //$NON-NLS-1$
 			}
@@ -337,6 +341,24 @@ public class StringUtils {
 		return 0;
 	}
 
+	public static char firstAlphaCharUc(String searchIn, int startAt) {
+		if (searchIn == null) {
+			return 0;
+		}
+
+		int length = searchIn.length();
+
+		for (int i = startAt; i < length; i++) {
+			char c = searchIn.charAt(i);
+
+			if (Character.isLetter(c)) {
+				return Character.toUpperCase(c);
+			}
+		}
+
+		return 0;
+	}
+	
 	/**
 	 * Adds '+' to decimal numbers that are positive (MySQL doesn't understand
 	 * them otherwise
@@ -449,7 +471,8 @@ public class StringUtils {
 	}
 
 	public static final byte[] getBytes(char[] c, String encoding,
-			String serverEncoding, boolean parserKnowsUnicode, Connection conn)
+			String serverEncoding, boolean parserKnowsUnicode, 
+			ConnectionImpl conn)
 			throws SQLException {
 		try {
 			
@@ -522,6 +545,48 @@ public class StringUtils {
 		}
 	}
 
+	public static final byte[] getBytesWrapped(String s, char beginWrap, char endWrap,
+			SingleByteCharsetConverter converter, String encoding,
+			String serverEncoding, boolean parserKnowsUnicode)
+			throws SQLException {
+		try {
+			byte[] b = null;
+
+			if (converter != null) {
+				b = converter.toBytesWrapped(s, beginWrap, endWrap);
+			} else if (encoding == null) {
+				StringBuffer buf = new StringBuffer(s.length() + 2);
+				buf.append(beginWrap);
+				buf.append(s);
+				buf.append(endWrap);
+				
+				b = buf.toString().getBytes();
+			} else {
+				StringBuffer buf = new StringBuffer(s.length() + 2);
+				buf.append(beginWrap);
+				buf.append(s);
+				buf.append(endWrap);
+				
+				b = buf.toString().getBytes(encoding);
+
+				if (!parserKnowsUnicode && (encoding.equalsIgnoreCase("SJIS") //$NON-NLS-1$
+						|| encoding.equalsIgnoreCase("BIG5") //$NON-NLS-1$
+				|| encoding.equalsIgnoreCase("GBK"))) { //$NON-NLS-1$
+
+					if (!encoding.equalsIgnoreCase(serverEncoding)) {
+						b = escapeEasternUnicodeByteStream(b, s, 0, s.length());
+					}
+				}
+			}
+
+			return b;
+		} catch (UnsupportedEncodingException uee) {
+			throw SQLError.createSQLException(Messages.getString("StringUtils.5") //$NON-NLS-1$
+					+ encoding + Messages.getString("StringUtils.6"),
+					SQLError.SQL_STATE_ILLEGAL_ARGUMENT); //$NON-NLS-1$
+		}
+	}
+	
 	/**
 	 * DOCUMENT ME!
 	 * 
@@ -606,7 +671,8 @@ public class StringUtils {
 	 *             if an encoding unsupported by the JVM is supplied.
 	 */
 	public static final byte[] getBytes(String s, String encoding,
-			String serverEncoding, boolean parserKnowsUnicode, Connection conn)
+			String serverEncoding, boolean parserKnowsUnicode, 
+			ConnectionImpl conn)
 			throws SQLException {
 		try {
 			SingleByteCharsetConverter converter = null;
@@ -705,16 +771,20 @@ public class StringUtils {
 	}
 
 	public static long getLong(byte[] buf) throws NumberFormatException {
+		return getLong(buf, 0, buf.length);
+	}
+	
+	public static long getLong(byte[] buf, int offset, int endpos) throws NumberFormatException {
 		int base = 10;
 
-		int s = 0;
+		int s = offset;
 
 		/* Skip white space. */
-		while (Character.isWhitespace((char) buf[s]) && (s < buf.length)) {
+		while (Character.isWhitespace((char) buf[s]) && (s < endpos)) {
 			++s;
 		}
 
-		if (s == buf.length) {
+		if (s == endpos) {
 			throw new NumberFormatException(new String(buf));
 		}
 
@@ -741,7 +811,7 @@ public class StringUtils {
 		boolean overflow = false;
 		long i = 0;
 
-		for (; s < buf.length; s++) {
+		for (; s < endpos; s++) {
 			char c = (char) buf[s];
 
 			if (Character.isDigit(c)) {
@@ -935,7 +1005,7 @@ public class StringUtils {
 			boolean allowBackslashEscapes) {
 		char contextMarker = Character.MIN_VALUE;
 		boolean escaped = false;
-		int markerTypeFound = -1;
+		int markerTypeFound = 0;
 		int srcLength = src.length();
 		int ind = 0;
 
@@ -944,9 +1014,8 @@ public class StringUtils {
 
 			if (allowBackslashEscapes && c == '\\') {
 				escaped = !escaped;
-			} else if (markerTypeFound != -1 && c == markerCloses.charAt(markerTypeFound) && !escaped) {
+			} else if (c == markerCloses.charAt(markerTypeFound) && !escaped) {
 				contextMarker = Character.MIN_VALUE;
-				markerTypeFound = -1;
 			} else if ((ind = marker.indexOf(c)) != -1 && !escaped
 					&& contextMarker == Character.MIN_VALUE) {
 				markerTypeFound = ind;
@@ -1431,7 +1500,7 @@ public class StringUtils {
 				: WILD_COMPARE_MATCH_NO_WILD);
 	}
 	
-	static byte[] s2b(String s, Connection conn) throws SQLException {
+	static byte[] s2b(String s, ConnectionImpl conn) throws SQLException {
 		if (s == null) {
 			return null;
 		}
@@ -1459,7 +1528,7 @@ public class StringUtils {
 
 		return s.getBytes();
 	}
-
+	
 	public static int lastIndexOf(byte[] s, char c) {
 		if (s == null) {
 			return -1;
@@ -1490,6 +1559,10 @@ public class StringUtils {
 		return -1;
 	}
 	
+	public static boolean isNullOrEmpty(String toTest) {
+		return (toTest == null || toTest.length() == 0);
+	}
+	
 	/**
 	 * Returns the given string, with comments removed
 	 * 
@@ -1618,4 +1691,20 @@ public class StringUtils {
 
 		return buf.toString();
 	}
+	
+	public static final boolean isEmptyOrWhitespaceOnly(String str) {
+		if (str == null || str.length() == 0) {
+			return true;
+		}
+		
+		int length = str.length();
+		
+		for (int i = 0; i < length; i++) {
+			if (!Character.isWhitespace(str.charAt(i))) {
+				return false;
+			}
+		}
+		
+		return true;
+	}
 }
