@@ -33,7 +33,6 @@ import java.util.Calendar;
 import java.util.Collections;
 import java.util.GregorianCalendar;
 import java.util.HashMap;
-import java.util.Locale;
 import java.util.Map;
 import java.util.TimeZone;
 
@@ -809,7 +808,7 @@ public class TimeUtil {
 	 * 
 	 * @return the times changed to the timezone 'toTz'
 	 */
-	public static Time changeTimezone(Connection conn,
+	public static Time changeTimezone(ConnectionImpl conn,
 			Calendar sessionCalendar, 
 			Calendar targetCalendar, 
 			Time t, 
@@ -871,7 +870,7 @@ public class TimeUtil {
 	 * 
 	 * @return the timestamp changed to the timezone 'toTz'
 	 */
-	public static Timestamp changeTimezone(Connection conn, 
+	public static Timestamp changeTimezone(ConnectionImpl conn, 
 			Calendar sessionCalendar, 
 			Calendar targetCalendar, 
 			Timestamp tstamp,
@@ -1153,7 +1152,6 @@ public class TimeUtil {
 	
 	private static String timeFormattedString(int hours, int minutes, int seconds) {
 		StringBuffer buf = new StringBuffer(8);
-		
 		if (hours < 10) {
 			buf.append("0");
 		}
