@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2002-2004 MySQL AB
+ Copyright (C) 2002-2007 MySQL AB
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of version 2 of the GNU General Public License as 
@@ -24,8 +24,11 @@
  */
 package com.mysql.jdbc.jdbc2.optional;
 
+import com.mysql.jdbc.ConnectionImpl;
 import com.mysql.jdbc.SQLError;
+import com.mysql.jdbc.Util;
 
+import java.lang.reflect.Constructor;
 import java.sql.Connection;
 import java.sql.ResultSet;
 import java.sql.SQLException;
@@ -42,11 +45,47 @@ import java.sql.Statement;
  *          Exp $
  */
 public class StatementWrapper extends WrapperBase implements Statement {
+	private static final Constructor JDBC_4_STATEMENT_WRAPPER_CTOR;
+	
+	static {
+		if (Util.isJdbc4()) {
+			try {
+				JDBC_4_STATEMENT_WRAPPER_CTOR = Class.forName(
+						"com.mysql.jdbc.jdbc2.optional.JDBC4StatementWrapper").getConstructor(
+						new Class[] { ConnectionWrapper.class, 
+								MysqlPooledConnection.class, 
+								Statement.class });
+			} catch (SecurityException e) {
+				throw new RuntimeException(e);
+			} catch (NoSuchMethodException e) {
+				throw new RuntimeException(e);
+			} catch (ClassNotFoundException e) {
+				throw new RuntimeException(e);
+			}
+		} else {
+			JDBC_4_STATEMENT_WRAPPER_CTOR = null;
+		}
+	}
+	
+	protected static StatementWrapper getInstance(ConnectionWrapper c, 
+			MysqlPooledConnection conn,
+			Statement toWrap) throws SQLException {
+		if (!Util.isJdbc4()) {
+			return new StatementWrapper(c, 
+					conn, toWrap);
+		}
+
+		return (StatementWrapper) Util.handleNewInstance(
+				JDBC_4_STATEMENT_WRAPPER_CTOR,
+				new Object[] {c, 
+						conn, toWrap });
+	}
+	
 	protected Statement wrappedStmt;
 
 	protected ConnectionWrapper wrappedConn;
 
-	protected StatementWrapper(ConnectionWrapper c, MysqlPooledConnection conn,
+	public StatementWrapper(ConnectionWrapper c, MysqlPooledConnection conn,
 			Statement toWrap) {
 		this.pooledConnection = conn;
 		this.wrappedStmt = toWrap;
@@ -64,7 +103,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedConn;
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -86,7 +125,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 			if (this.wrappedStmt != null) {
 				this.wrappedStmt.setCursorName(name);
 			} else {
-				throw new SQLException("Statement already closed",
+				throw SQLError.createSQLException("Statement already closed",
 						SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 			}
 		} catch (SQLException sqlEx) {
@@ -104,7 +143,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 			if (this.wrappedStmt != null) {
 				this.wrappedStmt.setEscapeProcessing(enable);
 			} else {
-				throw new SQLException("Statement already closed",
+				throw SQLError.createSQLException("Statement already closed",
 						SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 			}
 		} catch (SQLException sqlEx) {
@@ -122,7 +161,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 			if (this.wrappedStmt != null) {
 				this.wrappedStmt.setFetchDirection(direction);
 			} else {
-				throw new SQLException("Statement already closed",
+				throw SQLError.createSQLException("Statement already closed",
 						SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 			}
 		} catch (SQLException sqlEx) {
@@ -141,7 +180,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.getFetchDirection();
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -163,7 +202,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 			if (this.wrappedStmt != null) {
 				this.wrappedStmt.setFetchSize(rows);
 			} else {
-				throw new SQLException("Statement already closed",
+				throw SQLError.createSQLException("Statement already closed",
 						SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 			}
 		} catch (SQLException sqlEx) {
@@ -182,7 +221,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.getFetchSize();
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -204,7 +243,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.getGeneratedKeys();
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -226,7 +265,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 			if (this.wrappedStmt != null) {
 				this.wrappedStmt.setMaxFieldSize(max);
 			} else {
-				throw new SQLException("Statement already closed",
+				throw SQLError.createSQLException("Statement already closed",
 						SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 			}
 		} catch (SQLException sqlEx) {
@@ -245,7 +284,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.getMaxFieldSize();
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -266,7 +305,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 			if (this.wrappedStmt != null) {
 				this.wrappedStmt.setMaxRows(max);
 			} else {
-				throw new SQLException("Statement already closed",
+				throw SQLError.createSQLException("Statement already closed",
 						SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 			}
 		} catch (SQLException sqlEx) {
@@ -285,7 +324,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.getMaxRows();
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -307,7 +346,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.getMoreResults();
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -327,7 +366,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.getMoreResults(current);
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -346,7 +385,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 			if (this.wrappedStmt != null) {
 				this.wrappedStmt.setQueryTimeout(seconds);
 			} else {
-				throw new SQLException("Statement already closed",
+				throw SQLError.createSQLException("Statement already closed",
 						SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 			}
 		} catch (SQLException sqlEx) {
@@ -365,7 +404,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.getQueryTimeout();
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -384,12 +423,12 @@ public class StatementWrapper extends WrapperBase implements Statement {
 			if (this.wrappedStmt != null) {
 				ResultSet rs = this.wrappedStmt.getResultSet();
 
-				((com.mysql.jdbc.ResultSet) rs).setWrapperStatement(this);
+				((com.mysql.jdbc.ResultSetInternalMethods) rs).setWrapperStatement(this);
 
 				return rs;
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -409,7 +448,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.getResultSetConcurrency();
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -429,7 +468,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.getResultSetHoldability();
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -449,7 +488,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.getResultSetType();
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -469,7 +508,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.getUpdateCount();
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -489,7 +528,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.getWarnings();
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -588,7 +627,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.execute(sql, autoGeneratedKeys);
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -611,7 +650,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.execute(sql, columnIndexes);
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -635,7 +674,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.execute(sql, columnNames);
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -658,7 +697,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.execute(sql);
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -681,7 +720,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.executeBatch();
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -703,12 +742,12 @@ public class StatementWrapper extends WrapperBase implements Statement {
 			if (this.wrappedStmt != null) {
 
 				ResultSet rs = this.wrappedStmt.executeQuery(sql);
-				((com.mysql.jdbc.ResultSet) rs).setWrapperStatement(this);
+				((com.mysql.jdbc.ResultSetInternalMethods) rs).setWrapperStatement(this);
 
 				return rs;
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -732,7 +771,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.executeUpdate(sql, autoGeneratedKeys);
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -755,7 +794,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.executeUpdate(sql, columnIndexes);
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -779,7 +818,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.executeUpdate(sql, columnNames);
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -801,7 +840,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				return this.wrappedStmt.executeUpdate(sql);
 			}
 
-			throw new SQLException("Statement already closed",
+			throw SQLError.createSQLException("Statement already closed",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
 		} catch (SQLException sqlEx) {
 			checkAndFireConnectionError(sqlEx);
@@ -818,7 +857,7 @@ public class StatementWrapper extends WrapperBase implements Statement {
 				((com.mysql.jdbc.Statement) this.wrappedStmt)
 						.enableStreamingResults();
 			} else {
-				throw new SQLException(
+				throw SQLError.createSQLException(
 						"No operations allowed after statement closed",
 						SQLError.SQL_STATE_GENERAL_ERROR);
 			}
