@@ -45,6 +45,7 @@ import java.util.StringTokenizer;
 
 import testsuite.BaseTestCase;
 
+import com.mysql.jdbc.ConnectionImpl;
 import com.mysql.jdbc.Driver;
 import com.mysql.jdbc.NonRegisteringDriver;
 import com.mysql.jdbc.ReplicationConnection;
@@ -711,7 +712,7 @@ public class ConnectionRegressionTest extends BaseTestCase {
 		props.setProperty("maxReconnects", "1");
 
 		Connection failoverConnection = null;
-		Connection killerConnection = getConnectionWithProps((Properties)null);
+		Connection killerConnection = getConnectionWithProps((String)null);
 
 		try {
 			failoverConnection = getConnectionWithProps("jdbc:mysql://"
@@ -1238,8 +1239,8 @@ public class ConnectionRegressionTest extends BaseTestCase {
 
 			try {
 				replConn = getMasterSlaveReplicationConnection();
-				assertTrue(!((ReplicationConnection) replConn)
-						.getMasterConnection().hasSameProperties(
+				assertTrue(!((ConnectionImpl) ((ReplicationConnection) replConn)
+						.getMasterConnection()).hasSameProperties(
 								((ReplicationConnection) replConn)
 										.getSlavesConnection()));
 			} finally {
@@ -2072,24 +2073,21 @@ public class ConnectionRegressionTest extends BaseTestCase {
 	public void testFailoverReadOnly() throws Exception {
 		Properties props = getMasterSlaveProps();
 		props.setProperty("autoReconnect", "true");
-		
-		
+	
 		Connection failoverConn = null;
-		
-		
+
 		Statement failoverStmt = 
 			null;
 		
 		try {
 			failoverConn = getConnectionWithProps(getMasterSlaveUrl(), props);
-
+			
 			((com.mysql.jdbc.Connection)failoverConn).setPreferSlaveDuringFailover(true);
 			
 			failoverStmt = failoverConn.createStatement();
 			
 			String masterConnectionId = getSingleIndexedValueWithQuery(failoverConn, 1, "SELECT connection_id()").toString();
 			
-			
 			this.stmt.execute("KILL " + masterConnectionId);
 			
 			// die trying, so we get the next host
@@ -2142,6 +2140,27 @@ public class ConnectionRegressionTest extends BaseTestCase {
 		}
 	}
 	
+	public void testPropertiesDescriptionsKeys() throws Exception {
+		DriverPropertyInfo[] dpi = new NonRegisteringDriver().getPropertyInfo(
+				dbUrl, null);
+
+		for (int i = 0; i < dpi.length; i++) {
+			String description = dpi[i].description;
+			String propertyName = dpi[i].name;
+
+			if (description.indexOf("Missing error message for key '") != -1
+					|| description.startsWith("!")) {
+				fail("Missing message for configuration property "
+						+ propertyName);
+			}
+
+			if (description.length() < 10) {
+				fail("Suspiciously short description for configuration property "
+						+ propertyName);
+			}
+		}
+	}
+	
 	public void testBug29852() throws Exception {
     	Connection lbConn = getLoadBalancedConnection();
     	assertTrue(!lbConn.getClass().getName().startsWith("com.mysql.jdbc"));
@@ -2230,4 +2249,4 @@ public class ConnectionRegressionTest extends BaseTestCase {
 			closeMemberJDBCResources();
 		}
 	}
-}
\ No newline at end of file
+}
