@@ -39,6 +39,7 @@ import java.util.Properties;
 import testsuite.BaseTestCase;
 
 import com.mysql.jdbc.Driver;
+import com.mysql.jdbc.Field;
 import com.mysql.jdbc.NonRegisteringDriver;
 import com.mysql.jdbc.SQLError;
 
@@ -592,7 +593,7 @@ public class MetaDataRegressionTest extends BaseTestCase {
 	public void testBug4138() throws Exception {
 		try {
 			String[] typesToTest = new String[] { "TINYINT", "SMALLINT",
-					"MEDIUMINT", "INTEGER", "BIGINT", "FLOAT", "DOUBLE",
+					"MEDIUMINT", "INT", "BIGINT", "FLOAT", "DOUBLE",
 					"DECIMAL" };
 
 			short[] jdbcMapping = new short[] { Types.TINYINT, Types.SMALLINT,
@@ -1541,7 +1542,7 @@ public class MetaDataRegressionTest extends BaseTestCase {
 
 		assertEquals(true, dbmd.supportsGroupByUnrelated());
 	}
-	
+
 	/**
 	 * Tests fix for BUG#21267, ParameterMetaData throws NullPointerException
 	 * when prepared SQL actually has a syntax error
@@ -1601,8 +1602,8 @@ public class MetaDataRegressionTest extends BaseTestCase {
 		}
 		
 		createTable("testBug21544",
-                "(foo_id INT NOT NULL, stuff LONGTEXT"
-                + ", PRIMARY KEY (foo_id)) TYPE=INNODB");
+	            "(foo_id INT NOT NULL, stuff LONGTEXT"
+	            + ", PRIMARY KEY (foo_id)) TYPE=INNODB");
 		
 		Connection infoSchemConn = null;
 		
@@ -1621,12 +1622,12 @@ public class MetaDataRegressionTest extends BaseTestCase {
 	        	rs.getInt("COLUMN_SIZE");   
 	        }
 	    } finally {
-            if (infoSchemConn != null) {
-            	infoSchemConn.close();
-            }
-            
-            closeMemberJDBCResources();
-        }
+	        if (infoSchemConn != null) {
+	        	infoSchemConn.close();
+	        }
+	        
+	        closeMemberJDBCResources();
+	    }
 	}
 
 	/** 
@@ -1703,7 +1704,7 @@ public class MetaDataRegressionTest extends BaseTestCase {
 		
 		assertTrue(foundHost);
 	}
-
+	
 	private void testAbsenceOfMetadataForQuery(String query) throws Exception {
 		try {
 			this.pstmt = this.conn.prepareStatement(query);
@@ -1766,6 +1767,7 @@ public class MetaDataRegressionTest extends BaseTestCase {
 		this.conn.getMetaData().getProcedures(bogusDatabaseName, "%", "%");
 	}
 
+
 	/**
 	 * Tests fix for BUG#23303 - DBMD.getSchemas() doesn't return a TABLE_CATALOG column.
 	 * 
@@ -1792,7 +1794,6 @@ public class MetaDataRegressionTest extends BaseTestCase {
 	 * 
 	 * @throws Exception
 	 */
-	
 	public void testBug23304() throws Exception {
 		if (!versionMeetsMinimum(5, 0)) {
 			return;
@@ -1928,7 +1929,7 @@ public class MetaDataRegressionTest extends BaseTestCase {
 			fail(messageBuf.toString());
 		}
 	}
-	
+
 	/**
 	 * Tests fix for BUG#25624 - Whitespace surrounding storage/size specifiers in stored procedure
 	 * declaration causes NumberFormatException to be thrown when calling stored procedure.
@@ -1951,4 +1952,119 @@ public class MetaDataRegressionTest extends BaseTestCase {
 
 		this.conn.prepareCall("{call testBug25624(?,?)}").close();
 	}
+
+	
+	/**
+	 * Tests fix for BUG#27867 - Schema objects with identifiers other than
+	 * the connection character aren't retrieved correctly in ResultSetMetadata.
+	 * 
+	 * @throws Exception if the test fails.
+	 */
+	public void testBug27867() throws Exception {
+		try {
+			String gbkColumnName = "\u00e4\u00b8\u00ad\u00e6\u2013\u2021\u00e6\u00b5\u2039\u00e8\u00af\u2022";
+			createTable("ColumnNameEncoding", "(" + "`" + gbkColumnName
+					+ "` varchar(1) default NULL,"
+					+ "`ASCIIColumn` varchar(1) default NULL"
+					+ ")ENGINE=MyISAM DEFAULT CHARSET=utf8");
+			
+			this.rs = this.stmt
+					.executeQuery("SELECT * FROM ColumnNameEncoding");
+			java.sql.ResultSetMetaData tblMD = this.rs.getMetaData();
+
+			assertEquals(gbkColumnName, tblMD.getColumnName(1));
+			assertEquals("ASCIIColumn", tblMD.getColumnName(2));
+		} finally {
+			closeMemberJDBCResources();
+		}
+	}
+	
+	/**
+	 * Fixed BUG#27915 - DatabaseMetaData.getColumns() doesn't
+	 * contain SCOPE_* or IS_AUTOINCREMENT columns.
+	 * 
+	 * @throws Exception
+	 */
+	public void testBug27915() throws Exception {
+		createTable("testBug27915",
+				"(field1 int not null primary key auto_increment, field2 int)");
+		DatabaseMetaData dbmd = this.conn.getMetaData();
+
+		try {
+			this.rs = dbmd.getColumns(this.conn.getCatalog(), null,
+					"testBug27915", "%");
+			this.rs.next();
+
+			checkBug27915();
+
+			if (versionMeetsMinimum(5, 0)) {
+				this.rs = getConnectionWithProps("useInformationSchema=true")
+						.getMetaData().getColumns(this.conn.getCatalog(), null,
+								"testBug27915", "%");
+				this.rs.next();
+
+				checkBug27915();
+			}
+		} finally {
+			closeMemberJDBCResources();
+		}
+	}
+
+	private void checkBug27915() throws SQLException {
+		assertNull(this.rs.getString("SCOPE_CATALOG"));
+		assertNull(this.rs.getString("SCOPE_SCHEMA"));
+		assertNull(this.rs.getString("SCOPE_TABLE"));
+		assertNull(this.rs.getString("SOURCE_DATA_TYPE"));
+		assertEquals("YES", this.rs.getString("IS_AUTOINCREMENT"));
+
+		this.rs.next();
+		
+		assertNull(this.rs.getString("SCOPE_CATALOG"));
+		assertNull(this.rs.getString("SCOPE_SCHEMA"));
+		assertNull(this.rs.getString("SCOPE_TABLE"));
+		assertNull(this.rs.getString("SOURCE_DATA_TYPE"));
+		assertEquals("NO", this.rs.getString("IS_AUTOINCREMENT"));
+	}
+	
+	/**
+	 * Tests fix for BUG#27916 - UNSIGNED types not reported
+	 * via DBMD.getTypeInfo(), and capitalization of types is
+	 * not consistent between DBMD.getColumns(), RSMD.getColumnTypeName()
+	 * and DBMD.getTypeInfo().
+	 * 
+	 * This fix also ensures that the precision of UNSIGNED MEDIUMINT
+	 * and UNSIGNED BIGINT is reported correctly via DBMD.getColumns().
+	 * 
+	 * @throws Exception
+	 */
+	public void testBug27916() throws Exception {
+		createTable(
+				"testBug27916",
+				"(field1 TINYINT UNSIGNED, field2 SMALLINT UNSIGNED, field3 INT UNSIGNED, field4 INTEGER UNSIGNED, field5 MEDIUMINT UNSIGNED, field6 BIGINT UNSIGNED)");
+
+		ResultSetMetaData rsmd = this.stmt.executeQuery(
+				"SELECT * FROM testBug27916").getMetaData();
+
+		HashMap typeNameToPrecision = new HashMap();
+		this.rs = this.conn.getMetaData().getTypeInfo();
+
+		while (this.rs.next()) {
+			typeNameToPrecision.put(this.rs.getString("TYPE_NAME"), this.rs
+					.getObject("PRECISION"));
+		}
+
+		this.rs = this.conn.getMetaData().getColumns(this.conn.getCatalog(),
+				null, "testBug27916", "%");
+
+		for (int i = 0; i < rsmd.getColumnCount(); i++) {
+			this.rs.next();
+			String typeName = this.rs.getString("TYPE_NAME");
+
+			assertEquals(typeName, rsmd.getColumnTypeName(i + 1));
+			assertEquals(typeName, this.rs.getInt("COLUMN_SIZE"), rsmd
+					.getPrecision(i + 1));
+			assertEquals(typeName, new Integer(rsmd.getPrecision(i + 1)),
+					typeNameToPrecision.get(typeName));
+		}
+	}
 }
